#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include "console.h"
#include "uimage.h"
#include "logging.h"
#include "usplash.h"

static void set8(unsigned char *s, int c, int n)
{
  unsigned long *t = (unsigned long *)s;
  int a = (c << 24) | (c << 16) | (c << 8) | c;
  while(n > 3)
    {
      *t++ = a;
      n -= 4;
    }
  s = (unsigned char *)t;
  while(n-- > 0)
    *s++ = c;
}

static void set16(unsigned short *s, int c, int n)
{
  unsigned long *t = (unsigned long *)s;
  int a = c | (c << 16);
  while(n > 1)
    {
      *t++ = a;
      n -= 2;
    }
  if(n > 0)
    (unsigned short *)t = c;
}

static void set24(unsigned long *s, int c, int n)
{
  unsigned char *t;
  int r = c >> 16, g = (c >> 8) & 0xff, b = c & 0xff;
  int d = (c << 8) | r, e = (c << 16) | (c >> 16), f = (b << 24) | c;
  while(n > 3)
    {
      *s++ = d;
      *s++ = e;
      *s++ = f;
      n -= 4;
    }
  t = (unsigned char *)s;
  while(n-- > 0)
    {
      *t++ = r;
      *t++ = g;
      *t++ = b;
    }
}

static const inline int colour_24to16(int c)
{
  return
    ((c & 0xf80000) >> 8) |
    ((c & 0x00fc00) >> 5) |
    ((c & 0x0000f8) >> 3);
}

static void set32(unsigned long *s, int c, int n)
{
  while(n-- > 0)
    *s++ = c;
}

int main(int argc, char **argv)
{
  int fb_len, length, fb_fd;
  uimage *fbimage;
  char *fb_mmap;
  int colour = 0x000000;

  if(argc == 2 && (colour = strtol(argv[1] + (*argv[1] == '#' ? 1 : 0), NULL, 16)))
    ;

  if (!graphical_framebuffer_enabled())
    return 1;
  if((fbimage = uimage_new()) == NULL)
    return 2;
  if (uimage_from_framebuffer(fbimage))
    return 3;
  if ((fb_len = get_framebuffer_length()) == 0)
    return 4;
  if((fb_mmap = mmap(NULL, fb_len, PROT_WRITE, MAP_SHARED, fb_fd = get_framebuffer_fd(), 0)) == MAP_FAILED)
    return 5;

  disable_cursor(SPLASH_CON);
  clear_console(SPLASH_CON);
  change_console(SPLASH_CON);

  length = fbimage->height*fbimage->width;

  switch(fbimage->depth)
    {
    case 8:
      set8(fb_mmap, colour & 0xff, length);
      break;
    case 15:
    case 16:
      set16((unsigned short *)fb_mmap, colour_24to16(colour), length);
      break;
    case 24:
      set24((unsigned long *)fb_mmap, colour & 0xffffff, length);
      break;
    case 32:
      set32((unsigned long *)fb_mmap, colour & 0xffffff, length);
      break;
    default:
    }

  munmap(fb_mmap, fb_len);

  return 0;
}
