#include <stdlib.h>

/* Shared */
#include "messaging.h"
#include "console.h"
#include "logging.h"

/* Other */
#include "usplash.h"
#include "uimage.h"
#include "usplash_child.h"

static uimage *pcximage, *fbimage;
static uimage *throbber[4] = { NULL, NULL, NULL, NULL };
static int throbber_frame = 0;

#define THROBBER_FRAMES 3

static void place_throbber()
{
  char filename[512] = "";
  int i;
  if(throbber[0] == NULL)
    {
      for(i = 0; i < THROBBER_FRAMES; i++)
	{
	  if ((throbber[i] = uimage_new()) == NULL)
	    return;
	  snprintf(filename, 512, "media/throbber/%d.pcx", i);
	  if (uimage_from_pcx(throbber[i], filename))
	    {
	      uimage_free(throbber[i]);
	      throbber[i] = NULL;
	      return;
	    }
	  if (uimage_convert_depth(throbber[i], fbimage->depth))
	    {
	      uimage_free(throbber[i]);
	      throbber[i] = NULL;
	      return;
	    }
	  logme("place_throbber() loaded frame %d", i);
	}
      throbber[i+1] = NULL;
    }
  //uimage_display_icon(throbber[throbber_frame], fbimage->width-ICON_SPACING-throbber[i]->width, 0+ICON_SPACING);
  //uimage_display_icon(throbber[throbber_frame], fbimage->width-ICON_SPACING-throbber[i]->width, 0+ICON_SPACING);
	  
  if(throbber[throbber_frame] != NULL)
    uimage_display_icon(throbber[throbber_frame], fbimage->width - throbber[throbber_frame]->width - ICON_SPACING, 10);
  logme("throbber_frame = %d; ", throbber_frame);
  if(++throbber_frame >= THROBBER_FRAMES)
    throbber_frame = 0;
}

static icon_x = 0, icon_y = 0;

static void place_icon(char *s)
{
  char filename[512] = "";
  uimage *icon;
  if ((icon = uimage_new()) == NULL)
    return;
  snprintf(filename, 512, "media/daemons/%s.pcx", s);
  if (uimage_from_pcx(icon, filename))
    {
      logme("uimage_from_pcx(\"%s\") failed", s);
      return;
    }
  if (uimage_convert_depth(icon, fbimage->depth))
    return;
  if(icon_x > (fbimage->width - icon->width - ICON_SPACING))
    icon_x = ICON_SPACING, icon_y -= (ICON_SIZE + ICON_SPACING);

  logme("place_icon(\"%s\", %d, %d)", s, icon_x, icon_y);

  uimage_display_icon(icon, icon_x, icon_y);
  icon_x += icon->width + ICON_SPACING;
  uimage_free(icon);
}

/* Called when we receive an event */
void s_event_handler(s_event *event)
{
  int vt;
  logme("Received Event %s", se_to_string(event));
  if ((vt = get_current_console()) != SPLASH_CON)
    exit(vt);

  if (!strcmp(event->state, "exit"))
    {
      if ((vt = get_current_console()) == SPLASH_CON)
	{
	  uimage_display_on_fb(NULL, DISPTECH_FADEOUT);
	  exit(event->value);
	}
      else
	exit(vt);
    }
  else if (!strcmp(event->state, "attempt_run"))
    {
      place_icon(event->name);
    }
  else if (!strcmp(event->state, "percent"))
    {
      uimage_display_progress(event->value);
    }
}

int child_main()
{
  int t, loops = 0;
  /* Really want to save the text-console is possible */
  disable_cursor(SPLASH_CON);
  //clear_console(SPLASH_CON);
  change_console(SPLASH_CON);

  if ((pcximage = uimage_new()) == NULL)
    return -3;
  if ((fbimage = uimage_new()) == NULL)
    return -2;

  /* Get our images */
  if (uimage_from_framebuffer(fbimage))
    return -4;

  icon_x = ICON_SPACING;
  icon_y = fbimage->height - ICON_SIZE - ICON_SPACING;
  
  if (uimage_from_pcx(pcximage, "media/ubuntu-orange-466x400.pcx"))
    return -5;
  if (t = uimage_fill_to_size(pcximage, fbimage->width, fbimage->height))
    return -6;
  if (uimage_convert_depth(pcximage, fbimage->depth))
    return -7;
  //if (t = uimage_display_on_fb(pcximage, DISPTECH_CUT))
  if (t = uimage_display_on_fb(pcximage, DISPTECH_CROSSFADE))
    logme("uimage_display_on_fb() failed with %d", t);
  if (uimage_from_pcx(pcximage, "media/apache-163x80.pcx"))
    return -8;
  if (uimage_convert_depth(pcximage, fbimage->depth))
    return -9;

  /* Main loop */
  while (1) {
    int vt;
    loops++;
    if ((vt = get_current_console()) != SPLASH_CON)
      exit(vt);
    usleep(100 * 1000);
    poll_se_server();
    if((loops % 5) == 0)
      place_throbber();

  }
}



