#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <sys/un.h>

#define BUFFER 1000

#include "../lib/messaging.h"

int send_attempt_run(char *service_name) {
	s_event *event;
	int i;
	
	event = se_create(VERSION, service_name, MSG_ATTEMPT_RUN, 0);
	if (!event)
		return 1;
	i = se_send(event, SOCKET_LOCATION);
	se_free(event);
	return 1;
}

int send_run_status(char *service_name, int exitstatus) {
	s_event *event;
	int i;
	
	if (exitstatus)
		event = se_create(VERSION, service_name, MSG_RUN_STATUS_FAILURE, 0);
	else
		event = se_create(VERSION, service_name, MSG_RUN_STATUS_SUCCESS, 0);
	if (!event)
		return 1;
	
	i = se_send(event, SOCKET_LOCATION);
	se_free(event);
	return i;
}

int send_percent(char *service_name, int percent) {
	s_event *event;
	int i;
	
	if (percent > 100)
		percent = 100;
	else if (percent < 0)
		percent = 0;
	
	event = se_create(VERSION, service_name, MSG_PERCENT, percent);
	if (!event)
		return 1;
	
	i = se_send(event, SOCKET_LOCATION);
	se_free(event);
	return i;
}

int send_exit(int tty) {
	s_event *event;
	int i;
	
	if (tty > MAX_VT)
		tty = MAX_VT;
	else if (tty < 0)
		tty = 0;
	
	event = se_create(VERSION, "", MSG_EXIT, tty);
	if (!event)
		return 1;
	
	i = se_send(event, SOCKET_LOCATION);
	se_free(event);
	return i;
}

void usage(char *progname) {
	printf("%s [command] [data]\n", progname);
	puts("Commands:");
	puts("\t--attempt-run\t<proc_name>");
	puts("\t--run-status\t<proc_name> <exitstatus>");
	puts("\t--percent\t<proc_name> <percent>");
	puts("\t--exit\t\t[<tty>]");
	puts("\nData:");
	puts("\t<proc_name>\tthe name of the process you are starting");
	puts("\t<exitstatus>\tthe exitstatus of a process");
	puts("\t<percent>\tthe percentage.");
	puts("\t<tty>\t\tthe tty # to switch to.  tty is optional.");
}

int main(int argc, char **argv) {
	int i, tempint;
	char *output;

	if (argc < 3)
		goto bad;
	
	for (i=1 ; i < argc ; i++) {
		if (!strcmp(argv[i], "--attempt-run")) {
			if (argc <= (i+1))
				goto bad;
			i++;
			return send_attempt_run(argv[i]);				
		}
		if (!strcmp(argv[i], "--run-status")) {
			if (argc <= (i+2))
				goto bad;
			i++;
			output = argv[i];
			i++;
			tempint = atoi(argv[i]);
			if ((tempint == 0) && strcmp("0", argv[i]))
				goto bad;
			return send_run_status(output, tempint);
		}
		if (!strcmp(argv[i], "--percent")) {
			if (argc <= (i+2))
				goto bad;
			i++;
			output = argv[i];
			i++;
			tempint = atoi(argv[i]);
			if ((tempint == 0) && strcmp("0", argv[i]))
				goto bad;
			return send_percent(output, tempint);	
		}
		if (!strcmp(argv[i], "--exit")) {
			if (argc > (i+1)) 
				if ((atoi(argv[i+1]) == 0) && strcmp("0", argv[i+1]))
					goto bad;
				else
					tempint = atoi(argv[++i]);
			else
				tempint = 0;
			return send_exit(tempint);
		}		
	}	
	
	bad:
		usage(argv[0]);
		return 1;
}
