#define MASTER_VT "/dev/tty0"

/* Determines whether a file descriptor is a console or not */
extern int is_a_console(int fd);

/* Returns the filediscriptor of the master virtual terminal
or negative on fail.  Dont forget to close it! */
extern int get_master_console_fd();

/* Opens console number 'console' and returns the file descriptor */
extern int get_console_fd(int console);

/* Returns the integer of the current console or negative on error */
extern int get_current_console();

/* Returns a filediscriptor for the current framebuffer device
or negative on error.  Don't forget to close it!  The FD is opened r+w */
extern int get_framebuffer_fd();

/* Checks to make sure we have a graphical framebuffer.
Returns 1 if graphical *is* enabled, 0 if not enabled */
extern int graphical_framebuffer_enabled();

/* Returns the length of the framebuffer memory or 0 for failure */
extern unsigned long get_framebuffer_length();

/* Returns a pointer to a (static) struct with info on the framebuffer. 
Returns NULL on fail. */
extern const struct fb_var_screeninfo *get_framebuffer_specs();

/* A printf implimentation that prints to the console number specified */
extern int con_printf(int console, char *format, ...);

/* A printf implimentation that prints to the current console device 
rather than /dev/console (which may be statically linked in the kernel) */
extern int cur_con_printf(char *format, ...);

/* Changes to a specific terminal.  Returns 0 on success, nonzero on fail. */
extern int change_console(int console);

/* Turns off the cursor on the current terminal */
extern int disable_cursor();

/* Turns on the cursor on the current terminal */
extern int enable_cursor();

void display_fbinfo(const struct fb_var_screeninfo *info);

int get_framebuffer_size(const struct fb_var_screeninfo *info);

void *mmap_framebuffer(void);
