#include <stdarg.h> /* For va_lists (printf emulation) */
#include <unistd.h> /* For write() */

#include "console.h"

#define LOG_CONSOLE 9
#define BUFFER 1000

/* Default is to log to LOG_CONSOLE */
static int logging_term_or_fd = LOG_CONSOLE;
static int log_to_term = 1;

void set_logging_fd(int fd) {
	if (fd < 0)
		return;
	
	logging_term_or_fd = fd;
	log_to_term = 0;
}

void set_logging_term(int console) {
	if (console < 0)
		return;
	logging_term_or_fd = console;
	log_to_term = 1;	
}

int logme(char *format, ...) {
	va_list ap;
	char string[BUFFER];
	int status;
	
	va_start(ap, format);
	if (vsnprintf(string, BUFFER, format, ap) < 0 ){
		va_end(ap);
		return;
	}
	va_end(ap);
	
	if (log_to_term)
		return con_printf(logging_term_or_fd, "%s%c", string, '\n');
	else {
		status = write(logging_term_or_fd, string, strlen(string));
		write(logging_term_or_fd, "\n", strlen("\n"));
		return status;
	}
}
