#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <wait.h>
#include <sys/stat.h>
#include <unistd.h>

#include "messaging.h"
#include "logging.h"
#include "mounting.h"

char *get_tmpfs_mountpoint() {
	int i;
	/* Need to do this as strlen() is not evaluated by the preprocessor */
	static char tmpfs_mp[sizeof(SOCKET_LOCATION)];
	
	tmpfs_mp[0] = '\0';
	i = (int)(strrchr(SOCKET_LOCATION, '/') - SOCKET_LOCATION);
	strncpy(tmpfs_mp, SOCKET_LOCATION, i);
	return tmpfs_mp;
}

char *get_proc_mountpoint() {
	/* Need to do this as strlen() is not evaluated by the preprocessor */
	static char proc_mp[sizeof(SOCKET_LOCATION) + sizeof(PROC_SUBDIR)];
	
	proc_mp[0] = '\0';
	strcpy(proc_mp, get_tmpfs_mountpoint());
	strcat(proc_mp, PROC_SUBDIR);
	return proc_mp;
}

int mkdir_p(char *dir) {
	char command[200];
	
	snprintf(command, 200, "/bin/mkdir -p %s", dir);
	return WEXITSTATUS(system(command));
}

int mount(char *fstype, char *src, char *dest) {
	char *command = NULL;
	int i;
	struct stat s;
	
	/* Build the command */
	i = strlen(fstype) + strlen(src) + strlen(dest) + strlen(MOUNT_COMMAND) + 1;
	command = malloc(sizeof(char) * i);
	if (!command)
		return -1;
	snprintf(command, i, MOUNT_COMMAND, fstype, src, dest);
	
	/* If the mountpoint doesn't exist, try to create it */
	if (stat(dest, &s)) {
		logme("Mounting directory (%s) does not exist, attempting to create...", dest);
		if (mkdir_p(dest)) {
			free(command);
			logme("Unable to create mounting directory, aborting!");
			return -1;
		}
		logme("Directory created sucessfully!");
	}
	/* If the mountpoint exists, but is not a directory, remove the file,
	then create the mountpoint */
	else if (!S_ISDIR(s.st_mode)) {
		logme("Mounting directory (%s) exists, but is a file...", dest);
		logme("Attempting to remove file...");
		if (remove(dest)) {
			logme("Removal failed! Aborting!");
			free(command);
			return -1;
		}
		logme("File removed sucessfully!");
		logme("Removal succeeded, attempting to create mounting directory...");
		if (mkdir_p(dest)) {
			free(command);
			logme("Unable to create mounting directory, aborting!");
			return -1;
		}
		logme("Directory created successfully!");
	}
	
	/* Run the command */
	i = system(command);
	free(command);
	return WEXITSTATUS(i);
}

int umount(char *dest) {
	char *command = NULL;
	int i;
	
	/* Build the command */
	i = strlen(dest) + strlen(UMOUNT_COMMAND) + 1;
	command = malloc(sizeof(char) * i);
	if (!command)
		return -1;
	snprintf(command, i, UMOUNT_COMMAND, dest);
	
	/* Run the command */
	i = system(command);
	free(command);
	return WEXITSTATUS(i);
}

int mount_tmpfs() {
	return mount("tmpfs", "tmpfs", get_tmpfs_mountpoint());
}

int umount_tmpfs() {
	return umount(get_tmpfs_mountpoint());
}

int mount_proc() {
	return mount("proc", "proc", get_proc_mountpoint());
}

int umount_proc() {
	return umount(get_proc_mountpoint());	
}
