#include <string.h> /* For memset() */
#include <unistd.h> /* For read() */

/* For lstat() */
#include <sys/types.h>
#include <stdlib.h>

#include "upcx.h"

/*
** Reads the first 128 bytes of a PCX headers, from an file
** descriptor, into memory.
** RETURN zero on success.
*/
int read_pcx_header(pcx *header, int fd)
{
  if(!lseek(fd, 0, SEEK_SET))
    if(read(fd, header, sizeof(pcx)) == sizeof(pcx))
      return 0;
  return -1;
}

/*
** Does the file descriptor point to a PCX file, which is of a
** suitable colour-depth (8-bit) for us to use?
** RETURN zero on success.
*/
int is_pcx_suitable(int fd)
{
  pcx header;
  if(!read_pcx_header(&header, fd))
    if(header.manufacturer == PCX_ZSOFT &&
       header.version >= PCX_WITH_PALETTE &&
       header.encoding == PCX_RLE &&
       header.bits_per_pixel == 8)
      return 0;
  return -1;
}

/*
** Takes a raw PCX RLE stream and decompresses it into the destination
** buffer, which must be big enough!
** RETURN zero on success
**
** PCX images are RLE (Run Length Encoded as follows:
**  if(top two bits are set)  // >= 0xc0
**    use bottom six bit (& 0x3f) as RLE count for next byte;
**  else // < 0xc0
**    copy one byte normally;
*/
void raw_pcx_decode(unsigned char *dest, unsigned char *src, int width, int height)
{
  int x, y, count;

  for(y = 0; y < height; y++)
    for(x = 0; x < width;)
      if(*src < 0xc0)
	x++, *dest++ = *src++;
      else
	{
	  count = *src++ & 0x3f;
	  memset(dest, *src++, count);
	  dest += count, x += count;
	}
}

