/* Paul Sladen, 2004
**
** A PCX file:
**   128 bytes of header
**   ... bytes pseudo-RLE compressed data
**   1 byte
**   768 bytes of RGB colour-map data
**
** The only way to find the palette is either to know how long the
** file is a seek to 768 bytes from the end; _or_ to decompress the
** whole image data, skip one pixel and then read the colourmap.
*/

typedef struct {
  unsigned char manufacturer;
  unsigned char version;
  unsigned char encoding;
  unsigned char bits_per_pixel;
  unsigned short xmin;
  unsigned short ymin;
  unsigned short xmax;
  unsigned short ymax;
  unsigned short xdpi;
  unsigned short ydpi;
  unsigned char colourmap[48];
  unsigned char reserved;
  unsigned char planes;
  unsigned short scanline_length;
  unsigned short palette_info;
  unsigned short xsize;
  unsigned short ysize;
  unsigned char fill[54];
  unsigned char data[0];
} pcx;

enum {
  PCX_ZSOFT = 10,
  PCX_RLE = 1,
  PCX_WITH_PALETTE = 2,
  PCX_COLOUR_MAP_LENGTH = 769
};

/*
** Reads the first 128 bytes of a PCX headers, from an file
** descriptor, into memory.
** RETURN zero on success.
*/
extern int read_pcx_header(pcx *header, int fd);


/*
** Does the file descriptor point to a PCX file, which is of a
** suitable colour-depth (8-bit) for us to use?
** RETURN zero on success.
*/
extern int is_pcx_suitable(int fd);


/*
** Takes a raw PCX RLE stream and decompresses it into the destination
** buffer, which must be big enough!
** RETURN zero on success
**
** PCX images are RLE (Run Length Encoded as follows:
**  if(top two bits are set)  // >= 0xc0
**    use bottom six bit (& 0x3f) as RLE count for next byte;
**  else // < 0xc0
**    copy one byte normally;
*/
extern void raw_pcx_decode(unsigned char *dest, unsigned char *src, int width, int height);
